from connection.SocketConnection import *
from protocol.receive.Request import *
from base.Logger import *
import time

class Server:

    """
        $Id: Server.py,v 1.1.1.1 2003/04/10 14:38:57 maro Exp $
        Connects to the gateway and listens for messages

        @author maro
    """

    STATE_RUNNING = "running"
    STATE_STOPPED = "stopped"

    m_listener = None
    m_host = None
    m_port = 0
    m_conn = None
    m_state = None
    m_log = None

    def __init__(self, host, port):
        """Constructor"""
        self.m_host = host
        self.m_port = port
        self.m_log = Logger()

    def run(self):
        """Connects to gateway and listens for messages"""
        self.m_state = self.STATE_RUNNING
        self.m_conn = SocketConnection(self.m_host, self.m_port)
        self.m_conn.addMessageListener(self)
        self.m_conn.listen()
        self.m_state = self.STATE_STOPPED

    def shutdown(self):
        """Stops the server and shuts down the socket"""
        print " Stopping server."
        self.m_conn.shutdown()
        while self.m_conn.getState() == self.m_conn.STATE_RUNNING:
            time.sleep(1)
            print "Waiting for socket to close"
        print " Server is stopped"

    def addRequestListener(self, listener):
        """Adds an instance of a class which callback methods are called
            when messages arrive from the gateway"""
        self.m_listener = listener

    def onMessageReceive(self, msg):
        """When a message arrives from the socket, the message is parsed by
            the Request class, and depending on request-type either
            'onSMSReceive' or 'onNotificationReceive' is called on the
            listener."""
        self.m_log.logDebug("Got from socket: \n%s" % msg)
        r = Request(msg)
        self.m_log.logDebug("Request-type: %s" % r.getType())
        if ( r.getType() == r.SMS ):
            self.m_listener.onSMSReceive(r.getRequest())
        elif ( r.getType() == r.NOTIFICATION ):
            self.m_listener.onNotificationReceive(r.getRequest())

    def sendResponse(self, response):
        """Sends a response back to the gateway"""
        msg = response.toXML()
        self.m_conn.send(msg)



