<?php
  
    //
    // Copyright (c) 2002-2003 Zyneo HB. (http://www.zyneo.com)
    // See LICENSE distributed with this package for more information.
    //

    include_once(_DOCROOT . "/base/Base.php");
    include_once(_DOCROOT . "/sdk/protocol/Sms.php");
    include_once(_DOCROOT . "/sdk/protocol/Notification.php");
    
    /**
     * $Id: Request.php,v 1.1.1.1 2003/03/27 10:27:42 maro Exp $
     *
     * This class represents a 'request' sent from the gateway.
     * A request can either be an incoming SMS or an incoming notification
     * for an SMS.
     * @package sdk.protocol.receive
     * @author Magnus Rosenquist, Zyneo
     */
    class Request
        extends Base
    {
        /** Constant for defining this request as an SMS */
        var $TYPE_SMS = "sms";
        
        /** Constant for defining this request as a Notification */
        var $TYPE_NOT = "notification";
        
        /** Request type */
        var $__type;
        /** Raw response data (XML) */
        var $__raw;
        /** The request object */
        var $__request;
        /** Current tag processing when parsing XML */
        var $__currentTag;
        
        /**
         * Creates a new Request object with the desired content ($input)
         * and parses the XML.
         * @param string $input xml string containing the request.
         */
        function Request($input)
        {
            $this->registerClass(get_class($this));
            $this->__raw = $input;
            $this->parse();
        }

        /**
         * Returns the request in it's raw XML-form 
         * @return string xml request
         */
        function getRawData()     { return $this->__raw; }        
        /**
         * Returns the request as the object contained in the request, which 
         * can be an Sms or a Notification. To find out which use 'getRequestType'.
         * @return mixed either an Sms object or a Notification
         * @see Sms
         * @see Notification
         */
        function getRequest()     { return $this->__request; }
        /**
         * Returns the type for this request, one of 'sms' and 'notification'
         * @return string the request type.
         */
        function getRequestType() { return $this->__type; }
        
        /**
         * Parses the incoming XML-request.
         */
        function parse()
        {
            $this->registerMethod("parse");
            $this->logDebug("Parsing: \n" .$this->__raw ."\n\n");
            $parser = xml_parser_create("UTF-8");
            xml_set_object($parser, $this)
                or die ("Could not add callback");
            xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, false)
                or die ("Could not set option");
            xml_set_element_handler($parser, "startElement", "endElement")
                or die ("Could not add element handlers");
            xml_set_character_data_handler($parser, "characterData")
                or die ("Could not add data handler");
            //
            $result = xml_parse($parser, $this->__raw);
            if ( ! $result )
            {
                $this->logFatal("Could not parse response!");
                $this->logFatal("Code: " . xml_get_error_code($parser));
                $this->logFatal("Error: " . xml_error_string(xml_get_error_code($parser)));
                $this->logFatal("At: " . xml_get_current_line_number($parser));
            }
            xml_parser_free($parser);
        }
        
        function startElement($parser, $name, $attrs)
        {
            $this->__currentTag = $name;
            if ( $name == "sms" )
            {
                $this->__type = $this->TYPE_SMS;
                $this->__request = new Sms();
            }
                
            if ( $name == "notification" )
            {
                $this->__type = $this->TYPE_NOT;
                $this->__request = new Notification();
            }
                
            $this->parseAttributes($attrs);
        }
        
        function endElement($parser, $name)
        {
            $this->__currentTag = "";
        }
        
        /**
         * Parses character data
         */
        function characterData($parser, $data)
        {
            if ( $this->__type == $this->TYPE_SMS )
                $this->charSMS($data);
            else if ( $this->__type == "notification" )
                $this->charNotification($data);
        }
                    
        /** 
         * Parses character data for an SMS
         */
        function charSMS($data)
        {
            switch ( $this->__currentTag )
            {
                case "message":
                    $this->__request->setMessage($data);
                    break;
                default:
                    break;
            }
        }
        
        
        /** 
         * Parses character data for a Notification, for now it is empty.
         */
        function charNotification($data)
        {
            
        }
        
        /**
         * Parses attributes for a tag.
         */
        function parseAttributes($attrs)
        {
            if ( $this->__type == $this->TYPE_SMS )
                $this->attrSMS($attrs);
            else if ( $this->__type == "notification" )
                $this->attrNot($attrs);
        }
        
        /**
         * Manages attributes concerning SMS
         */
        function attrSMS($attrs)
        {
            switch ( $this->__currentTag )
            {
                case "to":
                    $this->__request->setTo($attrs["address"]);
                    break;
                case "from":
                    $this->__request->setFrom($attrs["address"]);
                    break;
                case "receive":
                    $this->__request->setDelivered($attrs["date"]);
                    break;
                default:
                    break;
            }
        }
        
        /**
         * Manages attributes concerning Notifications
         */
        function attrNot($attrs)
        {
            switch ( $this->__currentTag )
            {
                case "message-id":
                    $this->__request->setMessageId($attrs["value"]);
                    break;
                case "type":
                    $this->__request->setNotificationType($attrs["value"]);
                    break;
                case "provider":
                    $this->__request->setProviderDate($attrs["date"]);
                    break;
                default:
                    break;
            }
        }
        
    }  
?>
